"""
ALTTPR API Wrapper
~~~~~~~~~~~~~~~~~~~
A basic wrapper for the ALTTPR.com API.
"""

__title__ = 'pyz3r'
__author__ = 'Thomas Prescott'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2018-2019 Thomas Prescott'
__version__ = '5.5.0'

from .alttpr import alttpr
from .sm import sm
from .smvaria import SuperMetroidVaria
from . import mystery, customizer, rom, patch